# ===========================================================================
#
# SYNOPSIS
#
#   AX_APPEND_HARDENING_FLAGS([YESNO])
#
# DESCRIPTION
#
#   Attempts to append hardening flags to HARDEN_CFLAGS, HARDEN_LDFLAGS,
#   HARDEN_CPPFLAGS.
#
#   YESNO - iff YESNO != no, then flags are appended via
#   AX_APPEND_{COMPILE,LINK}_FLAGS, AX_APPEND_FLAG, and
#   AX_CHECK_{COMPILE,LINK}_FLAG
#
# LICENSE
#
#   Copyright (c) 2013, pi-rho <ubuntu@tyr.cx>
#
#   This program is free software: you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the
#   Free Software Foundation, either version 3 of the License, or (at your
#   option) any later version.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
#   Public License for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program. If not, see <http://www.gnu.org/licenses/>.
#
#   As a special exception, the respective Autoconf Macro's copyright owner
#   gives unlimited permission to copy, distribute and modify the configure
#   scripts that are the output of Autoconf when processing the Macro. You
#   need not follow the terms of the GNU General Public License when using
#   or distributing such scripts, even though portions of the text of the
#   Macro appear in them. The GNU General Public License (GPL) does govern
#   all other use of the material that constitutes the Autoconf Macro.
#
#   This special exception to the GPL applies to versions of the Autoconf
#   Macro released by the Autoconf Archive. When you make and distribute a
#   modified version of the Autoconf Macro, you may extend this special
#   exception to the GPL to apply to your modified version as well.
#
# ===========================================================================

# serial 1

AC_DEFUN([AX_APPEND_HARDENING_FLAGS],[dnl
    AC_REQUIRE([AX_CHECK_COMPILE_FLAG])
    AC_REQUIRE([AX_CHECK_LINK_FLAG])
    AC_REQUIRE([AX_APPEND_FLAG])
    AC_REQUIRE([AX_APPEND_COMPILE_FLAGS])
    AC_REQUIRE([AX_APPEND_LINK_FLAGS])
    AS_IF([ test x"$1" != x"no" ], [
        HARDEN_CFLAGS=""
        HARDEN_CPPFLAGS=""
        HARDEN_LDFLAGS=""
        LINK_CHECK_FLAGS="-Werror"
        COMPILE_CHECK_FLAGS="-Werror"
        AX_APPEND_LINK_FLAGS([-Wl,--fatal-warnings],[LINK_CHECK_FLAGS])
        AX_CHECK_COMPILE_FLAG([-fstack-protector-all], [
            AX_CHECK_LINK_FLAG([-fstack-protector-all], [
                AX_APPEND_FLAG([-fstack-protector-all], [HARDEN_CFLAGS])
                AX_CHECK_COMPILE_FLAG([-Wstack-protector], [
                    AX_APPEND_FLAG([-Wstack-protector], [HARDEN_CFLAGS])], [], [-fstack-protector-all $COMPILE_CHECK_FLAGS])
                AX_CHECK_COMPILE_FLAG([--param=ssp-buffer-size=1], [
                    AX_APPEND_FLAG([--param=ssp-buffer-size=1], [HARDEN_CFLAGS])], [], [-fstack-protector-all $COMPILE_CHECK_FLAGS])
            ],[],[$LINK_CHECK_FLAGS])
        ],[],[$COMPILE_CHECK_FLAGS])
        AX_CHECK_COMPILE_FLAG([-fPIE], [
            AX_CHECK_LINK_FLAG([-fPIE -pie], [
                AX_APPEND_FLAG([-fPIE], [HARDEN_CFLAGS])
                AX_APPEND_FLAG([-pie], [HARDEN_LDFLAGS])
            ], [
                AX_CHECK_LINK_FLAG([-fPIE -Wl,-pie], [
                    AX_APPEND_FLAG([-fPIE], [HARDEN_CFLAGS])
                    AX_APPEND_FLAG([-Wl,-pie], [HARDEN_LDFLAGS])
                ],[].[$LINK_CHECK_FLAGS])
            ],[$LINK_CHECK_FLAGS])
        ],[].[$COMPILE_CHECK_FLAGS])
        AX_APPEND_FLAG([-D_FORTIFY_SOURCE=2], [HARDEN_CPPFLAGS])
        AX_APPEND_COMPILE_FLAGS([                         \
                                 -O2                      \
                                 -fno-strict-overflow     \
                                 -fno-common              \
                                 -Werror=format-security  \
                                ],
                                [HARDEN_CFLAGS],[$COMPILE_CHECK_FLAGS])
        AX_APPEND_LINK_FLAGS([                            \
                                 -Wl,-Bsymbolic-functions \
                                 -Wl,-z,nodlopen          \
                                 -Wl,-z,nodump            \
                                 -Wl,-z,noexecheap        \
                                 -Wl,-z,noexecstack       \
                                 -Wl,-z,relro             \
                                 -Wl,-z,now               \
                             ],
                             [HARDEN_LDFLAGS],
                             [$LINK_CHECK_FLAGS])
        AC_SUBST([HARDEN_CFLAGS])
        AC_SUBST([HARDEN_CPPFLAGS])
        AC_SUBST([HARDEN_LDFLAGS])
    ])dnl AS_IF($hardening)
])dnl AX_APPEND_HARDENING_FLAGS
