#ifndef __PCAP_H__
#define __PCAP_H__

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include "config.h"

#define MAGIC 0xa1b2c3d4
#define NGMARK 0x0a0d0d0a
#define NGMAGIC 0x1a2b3c4d
#define MAXFRAME 65535

/* Described at http://www.tcpdump.org/linktypes.html */
#define LINKTYPE_ETHERNET 1
#define LINKTYPE_RAW 101

struct pcap_file {
    FILE *f;
    const char *fn;
    uint32_t linktype;
    bool swap;
};

struct pcap_file_header {
    uint32_t magic;
    uint16_t version_major;
    uint16_t version_minor;
    int32_t  thiszone;          /* gmt to local correction */
    uint32_t sigfigs;           /* accuracy of timestamps */
    uint32_t snaplen;           /* max length saved portion of each pkt */
    uint32_t linktype;          /* data link type (LINKTYPE_*) */
};

struct pcap_pkthdr {
    struct pcap_timeval {
        uint32_t tv_sec;
        uint32_t tv_usec;
    } ts;                         /* time stamp */
    uint32_t   caplen;            /* length of portion present */
    uint32_t   len;               /* length this packet (off wire) */
};

struct linklayer {
    uint16_t           src[3];
    uint16_t           dest[3];
    uint16_t           ethertype;
};


#ifndef max
#  define max(a, b) ((a)>(b)?(a):(b))
#endif

#ifndef min
#  define min(a, b) ((a)<(b)?(a):(b))
#endif

#ifdef HAVE_BYTESWAP
#include <byteswap.h>
#   define bswap32(i) bswap_32(i)
#   define bswap16(i) bswap_16(i)
#endif

#ifndef bswap32
#define bswap32(i) ((((uint32_t)i & 0xff000000) >> 030) | \
                    (((uint32_t)i & 0x00ff0000) >> 010) | \
                    (((uint32_t)i & 0x0000ff00) << 010) | \
                    (((uint32_t)i & 0x000000ff) << 030))
#endif

#ifndef bswap16
#define bswap16(i) ((((uint16_t)i & 0xff00) >> 010) | \
                    (((uint16_t)i & 0x00ff) << 010))
#endif


int pcap_open_in(struct pcap_file *ctx, FILE *f);
int pcap_open_out(struct pcap_file *ctx, FILE *f);
int pcap_read_pkthdr(struct pcap_file *ctx, struct pcap_pkthdr *hdr);
int pcap_write_pkthdr(struct pcap_file *ctx, struct pcap_pkthdr *hdr);
void pcap_close(struct pcap_file *ctx);

#endif /* __PCAP_H__ */
