/*
 *   _ __ ___ _ __  _ __
 *  | '__/ _ \ '_ \| '__|
 *  | | |  __/ |_) | |
 *  |_|  \___| .__/|_|
 *           |_|
 *
 * Quote non-printable bytes, similar to python's repr()
 *
 */

#include <getopt.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <sysexits.h>
#include "netre.h"

int version(bool error) {
    fprintf(WHICHOUT(error), "repr v.%s - %s\n\n", PACKAGE_VERSION,
            "quote non-printable bytes, similar to python's repr()");
    return error ? EX_USAGE : EX_OK;
}

int usage(bool error, char *prog) {
    int retval = version(error);
    fprintf(WHICHOUT(error), "Usage: %s [-] [FILE ...]\n\n", prog);
    return retval;
}

int repr(FILE *fp, const char *fn) {
    int c;

    while ((c = getc(fp)) != EOF) {
        if (31 < c && c <  92) printf("%c", c)               ;  else
        if (     c ==       9) printf("\\t")                 ;  else
        if (     c ==      10) printf("\\n")                 ;  else
        if (     c ==      13) printf("\\r")                 ;  else
        if (     c ==      92) printf("\\\\")                ;  else
        if (92 < c && c < 127) printf("%c", c)               ;  else
        /*      defaults    */ printf("\\x%02x", (unsigned)c);
    }

    printf("\n");

    if (ferror(fp)) {
        perror(fn);
        clearerr(fp);
        return EX_IOERR;
    }
    if (ferror(stdout)) {
        perror("stdout");
        clearerr(stdout);
        return EX_IOERR;
    }

    return EX_OK;
}

int main(int argc, char *argv[]) {
    int   opt, retval;
    FILE       *input = stdin;
    const char  *infn = "stdin";

    /* option parsing */
    while ((opt = getopt(argc, argv, "hv")) != -1) {
        switch (opt) {
            case 'v': return version(false);
            case 'h': return usage(false, argv[0]);
            default:  return usage(true,  argv[0]);
        }
    }

    /* fast forward argv */
    argv += optind;

    do {
        if (*argv) {
            if (!strncmp(*argv, "-", 1)) {
                input = stdin;
            } else if ((input = fopen(*argv, "rb")) == NULL) {
                perror(*argv);
                argv++;
                continue;
            }
            infn = *argv++;
        }

        retval = repr(input, infn);

        if (input  != stdin) fclose(input);
        if (retval != EX_OK) return retval;
    } while (*argv);

    return EX_OK;
}
