#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include "stream.h"

void sinit(struct stream *s, char const *buf, size_t buflen, endianness endian) {
    s->buf = buf;
    s->len = buflen;
    s->endian = endian;
}

bool sskip(struct stream *s, size_t count) {
    if (count > s->len) {
        s->len = 0;
        return false;
    }

    s->buf += count;
    s->len -= count;
    return true;
}

bool sread(struct stream *s, void *buf, size_t count) {
    void const *d = s->buf;

    if (!sskip(s, count)) return false;
    memcpy(buf, d, count);

    return true;
}

uint8_t read_uint8(struct stream *s) {
    uint8_t *d = (uint8_t *)s->buf;

    if (!sskip(s, 1)) return 0;
    return d[0];
}

uint16_t read_uint16be(struct stream *s) {
    uint8_t *d = (uint8_t *)s->buf;

    if (!sskip(s, 2)) return 0;
    return (uint16_t)((d[0] << 8) | (d[1] << 0));
}

uint16_t read_uint16le(struct stream *s) {
    uint8_t *d = (uint8_t *)s->buf;

    if (!sskip(s, 2)) return 0;
    return (uint16_t)((d[0] << 0) | (d[1] << 8));
}

uint16_t read_uint16(struct stream *s) {
	if (s->endian == ENDIAN_BIG) {
		return read_uint16be(s);
	} else {
		return read_uint16le(s);
	}
}

uint32_t read_uint32be(struct stream *s) {
    uint8_t *d = (uint8_t *)s->buf;

    if (!sskip(s, 4)) return 0;
    return (uint32_t)((d[0] << 24) | (d[1] << 16) | (d[2] << 8) | (d[3] << 0));
}

uint32_t read_uint32le(struct stream *s) {
    uint8_t *d = (uint8_t *)s->buf;

    if (!sskip(s, 4)) return 0;
    return (uint32_t)((d[3] << 24) | (d[2] << 16) | (d[1] << 8) | (d[0] << 0));
}

uint32_t read_uint32(struct stream *s) {
	if (s->endian == ENDIAN_BIG) {
		return read_uint32be(s);
	} else {
		return read_uint32le(s);
	}
}

void mac_addr(char addr_s[], uint16_t addr[]) {
    int n;
    n = snprintf(addr_s, 15, "%04x.%04x.%04x", addr[0], addr[1], addr[2]);
    if (n >= 15) addr_s[14] = '\0';
}

void ip4_addr(char addr_s[], uint32_t addr) {
    int n;
    n = snprintf(addr_s, 16, "%u.%u.%u.%u",
                             (addr >> 24) & 0xff,
                             (addr >> 16) & 0xff,
                             (addr >> 8)  & 0xff,
                             (addr >> 0)  & 0xff);
    if (n >= 16) addr_s[15] = '\0';
}

void tcp_flags(char flags_s[], uint8_t flags) {
    int i, n;
    n = snprintf(flags_s, 9, "fsrpauec");
    if (n >= 9) flags_s[8] = '\0';

    for (i = 0; i < 8; i++)
        if (!CHECK_BIT(flags, i)) flags_s[i] = '.';
}

void tcp_options(char opts_s[], struct stream *s, uint8_t len) {
    int i, n;
    char *out = opts_s;

    n = snprintf(out, 2, "-");
    if (n >= 2) out[1] = '\0';

    for (i = 0; i < len; i++) {
        n = snprintf(out, 3, "%02x", read_uint8(s));
        if (n >= 3) out[2] = '\0';
        out += 2;
    }
}

