#ifndef _STREAM_H
#define _STREAM_H

#include <stdbool.h>
#include <stdint.h>
#include <stddef.h>

#define IPPROTO_ICMP 1
#define IPPROTO_TCP  6
#define IPPROTO_UDP 17
#define IP_VER(vhl)   ((vhl >> 4) & 0x0f)
#define IP_IHL_s(vhl) ((vhl & 0x0f) * 4)
#define IP_IHL(vhl)   (size_t)(IP_IHL_s(vhl) < 20 ? 20 : IP_IHL_s(vhl))

#define ICMP_HLEN 8
#define TCP_HLEN 20
#define UDP_HLEN 8

#define ARP_REQ 0x01
#define ARP_REP 0x02

#define TH_FIN 0x01
#define TH_SYN 0x02
#define TH_RST 0x04
#define TH_PSH 0x08
#define TH_ACK 0x10
#define TH_URG 0x20

#define CHECK_BIT(var,bit) ((var & (1<<bit)) != 0)

typedef enum {
    NETWORK_ORDER = 0,
    ENDIAN_BIG = 0,
    ENDIAN_LITTLE = 1,
} endianness;

struct stream {
    char const *buf;
    size_t len;
    endianness endian;
};

void sinit(struct stream *s, char const *buf, size_t len, endianness endian);
bool sskip(struct stream *s, size_t count);
bool sread(struct stream *s, void *buf, size_t count);
uint8_t read_uint8(struct stream *s);
uint16_t read_uint16be(struct stream *s);
uint16_t read_uint16le(struct stream *s);
uint16_t read_uint16(struct stream *s);
uint32_t read_uint32be(struct stream *s);
uint32_t read_uint32le(struct stream *s);
uint32_t read_uint32(struct stream *s);

void mac_addr(char addr_s[], uint16_t addr[]);
void ip4_addr(char addr_s[], uint32_t addr);
void tcp_flags(char flags_s[], uint8_t flags);
void tcp_options(char opts_s[], struct stream *s, uint8_t len);

#endif
