/*
 *               _
 *   _   _ _ __ | |__   _____  __
 *  | | | | '_ \| '_ \ / _ \ \/ /
 *  | |_| | | | | | | |  __/>  <
 *   \__,_|_| |_|_| |_|\___/_/\_\
 *
 * convert hexits to their binary equivalent
 *
 */

#include <err.h>
#include <getopt.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <sysexits.h>
#include "netre.h"

int version(bool error) {
    fprintf(WHICHOUT(error), "unhex v.%s - %s\n\n", PACKAGE_VERSION,
            "convert hexits to their binary equivalent");
    return error ? EX_USAGE : EX_OK;
}

int usage(bool error, char *prog) {
    int retval = version(error);
    fprintf(WHICHOUT(error), "Usage: %s [-] [FILE ...]\n\n", prog);
    return retval;
}

int unhex(FILE *fp, const char *fn) {
    unsigned char        acc = '\0';
    unsigned char     nybble = '\0';
    unsigned long int  count =   0;
    int c;

    while ((c = getc(fp)) != EOF) {
        count++;

               if ('0' <= c && c <= '9') {
            acc = (acc << 4) + (unsigned char)c - '\x30'; nybble++;
        } else if ('A' <= c && c <= 'F') {
            acc = (acc << 4) + (unsigned char)c - '\x37'; nybble++;
        } else if ('a' <= c && c <= 'f') {
            acc = (acc << 4) + (unsigned char)c - '\x57'; nybble++;
        } else {
            if (nybble != 0)
                warnx("Warning: non-hex character mid-octet at offset %lu", count);
        }

        if (nybble == 2) {
            putchar(acc);
            acc = nybble = 0;
        }
    }

    if (ferror(fp)) {
        perror(fn);
        clearerr(fp);
        return EX_IOERR;
    }

    if (ferror(stdout)) {
        perror("stdout");
        clearerr(stdout);
        return EX_IOERR;
    }

    return EX_OK;
}

int main(int argc, char *argv[]) {
    int   opt, retval;
    FILE       *input = stdin;
    const char    *fn = "stdin";


    /* option parsing */
    while ((opt = getopt(argc, argv, "hv")) != -1) {
        switch (opt) {
            case 'v': return version(false);
            case 'h': return usage(false, argv[0]);
            default:  return usage(true,  argv[0]);
        }
    }

    /* fast forward argv */
    argv += optind;

    do {
        if (*argv) {
            if (!strncmp(*argv, "-", 1)) {
                input = stdin;
            } else if ((input = fopen(*argv, "rb")) == NULL) {
                perror(*argv);
                argv++;
                continue;
            }
            fn = *argv++;
        }

        retval = unhex(input, fn);

        if (input  != stdin) fclose(input);
        if (retval != EX_OK) return retval;
    } while(*argv);

    return EX_OK;
}
